#include "pw/visual/mesh_renderer.hpp"

#include "pw/core/mesh.hpp"
#include "pw/core/size.hpp"

#include "glad/glad.h"

#include <algorithm>

namespace pw {


struct mesh_renderer::impl {

	GLuint _vao = 0;
	std::vector<GLuint> _vbos;

	impl()
	{
	}

	~impl()
	{
		destroy();
	}

	bool ready() const
	{
		return GL_TRUE == glIsVertexArray(_vao);
	}

	void create(const mesh& m)
	{
		glGenVertexArrays(1,&_vao);
		glBindVertexArray(_vao);

		size_t arrays_needed = 0;

		// should bail out here
		if (!m.vertices().empty()) arrays_needed++;
		if (!m.indices().empty()) arrays_needed++;

		// TODO: implement the other arrays

		_vbos.resize(arrays_needed);

		glGenBuffers(_vbos.size(), _vbos.data());

		// vertices
		glBindBuffer(GL_ARRAY_BUFFER, _vbos[0]);
		glBufferData(GL_ARRAY_BUFFER, sizeof(m.vertices().front()) * m.vertices().size(), m.vertices().data(),
					 GL_STATIC_DRAW);
		glEnableVertexAttribArray(0);
		glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, 0, nullptr);

		// indices
		glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, _vbos[1]);
		glBufferData(GL_ELEMENT_ARRAY_BUFFER, sizeof(m.indices().front()) * m.indices().size(), m.indices().data(),
				GL_STATIC_DRAW);

		// stop binding
		glBindVertexArray(0);

	}


	void destroy()
	{
		glDeleteVertexArrays(1,&_vao);
		for (auto vbo : _vbos)
			glDeleteBuffers(1,&vbo);

		_vbos.clear();
	}

    void draw()
    {
		glBindVertexArray(_vao);
		glDrawElements(GL_TRIANGLES, 3, GL_UNSIGNED_INT, nullptr);
    }

};

//
//
//

mesh_renderer::mesh_renderer()
	: _impl(std::make_unique<mesh_renderer::impl>())
{
}

mesh_renderer::~mesh_renderer()
{
}

bool mesh_renderer::ready() const
{
	return _impl->ready();
}

void mesh_renderer::create(const mesh &m)
{
	_impl->create(m);
}

void mesh_renderer::destroy()
{
	_impl->destroy();
}

void mesh_renderer::draw()
{
	_impl->draw();
}


}
